from observer import *
import time
import wave
import numpy as np
import scipy.io.wavfile as wav
from numpy.lib import stride_tricks
import pylab
import pyaudio

class Vo(Observer):
    FORMAT = pyaudio.paInt16
    CHANNELS = 1
    RATE = 44100
    CHUNK = 512
    RECORD_SECONDS = 5
    DEVICE_INDEX = 2
    WAVE_OUTPUT_FILENAME = "file.wav"
    Recode_Value = 1000
    bindo_Value = 0.6

    Cry_Value = 1000
    Cut_Value = 100
    Sick_Value = 50000
    Hungry_Value = 20000
    #Sleepy_Value = 0
    
    def __init__(self,Push):
        Observer.__init__(Observer, Push)

    def run(self):
        while (True):
            print("running vo")

            item = self.popRQ()
            if (item != False):
                print("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!vo:  %s request %s" % (item.user, item.msg))
                

            if (True):

                time.sleep(2)
                
                detection_flag = self.recode()
                if detection_flag:
                    self.detection2()

                
    def recode(self):
        audio = pyaudio.PyAudio()
        stream = audio.open(
            format=self.FORMAT,
            channels=self.CHANNELS,
            rate=self.RATE,
            input=True,
            input_device_index=self.DEVICE_INDEX,
            frames_per_buffer=self.CHUNK)
        frames = []


        data = stream.read(self.CHUNK, exception_on_overflow = False)
        check = pylab.fromstring(data, 'int16')
        if (sum(abs(check)) / len(check)) < self.Recode_Value:
            stream.stop_stream()
            stream.close()
            audio.terminate()
            return False
        print("recode...")
        for i in range(1, int(self.RATE / self.CHUNK * self.RECORD_SECONDS)):
            data = stream.read(self.CHUNK, exception_on_overflow = False)
            frames.append(data)

        stream.stop_stream()
        stream.close()
        audio.terminate()

        waveFile = wave.open(self.WAVE_OUTPUT_FILENAME, 'wb')
        waveFile.setnchannels(self.CHANNELS)
        waveFile.setsampwidth(audio.get_sample_size(self.FORMAT))
        waveFile.setframerate(self.RATE)
        waveFile.writeframes(b''.join(frames))
        waveFile.close()
        return True

    # short time fourier transform of audio signal
    def stft(self,sig, frameSize, overlapFac=0.5, window=np.hanning):
        # sig = signal frameSize = 2 ** 10

        win = window(frameSize)
        hopSize = int(frameSize - np.floor(overlapFac * frameSize))

        # zeros at beginning (thus center of 1st window should be for sample nr. 0)
        samples = np.append(np.zeros(int(np.floor(frameSize / 2.0))), sig)
        # cols for windowing
        cols = np.ceil((len(samples) - frameSize) / float(hopSize)) + 1
        # zeros at end (thus samples can be fully covered by frames)
        samples = np.append(samples, np.zeros(frameSize))

        frames = stride_tricks.as_strided(samples, shape=(int(cols), frameSize),
                                          strides=(samples.strides[0] * hopSize, samples.strides[0])).copy()
        frames *= win

        return np.fft.rfft(frames)

    # scale frequency axis logarithmically
    def logscale_spec(self,spec, sr=44100, factor=20.):
        timebins, freqbins = np.shape(spec)

        scale = np.linspace(0, 1, freqbins) ** factor
        scale *= (freqbins - 1) / max(scale)
        scale = np.unique(np.round(scale))

        # create spectrogram with new freq bins
        newspec = np.complex128(np.zeros([timebins, len(scale)]))
        for i in range(0, len(scale)):
            if i == len(scale) - 1:
                newspec[:, i] = np.sum(spec[:, int(scale[i]):], axis=1)
            else:
                newspec[:, i] = np.sum(spec[:, int(scale[i]):int(scale[i + 1])], axis=1)

        # list center freq of bins
        allfreqs = np.abs(np.fft.fftfreq(freqbins * 2, 1. / sr)[:freqbins + 1])
        freqs = []
        for i in range(0, len(scale)):
            if i == len(scale) - 1:
                freqs += [np.mean(allfreqs[int(scale[i]):])]
            else:
                freqs += [np.mean(allfreqs[int(scale[i]):int(scale[i + 1])])]

        return newspec, freqs

    def detection2(self):
        samplerate, samples = wav.read(self.WAVE_OUTPUT_FILENAME)
        binsize = 2 ** 10



        s = self.stft(samples, binsize)
        sshow, freq = self.logscale_spec(s, sr=samplerate, factor=1.0)

        ims = 20. * np.log10(np.abs(sshow) / 10e-6)
        timebins, freqbins = np.shape(ims)

        time = ((timebins * len(samples) / timebins) + (0.5 * binsize)) / samplerate
        timei = ((len(samples) / timebins) + (0.5 * binsize)) / samplerate
        timei /= 2

        maxnum = 0
        fr25 = 0
        tmp = 0
        fullcount = 0

        for i in range(timebins):
            for j in range(freqbins):
                if i == 0:
                    if freq[j] < 2500:
                        tmp = j
                if ims[i][j] < 200:
                    ims[i][j] = 0
                else:
                    maxnum += 1
                    fullcount += ims[i][j]

        ti = 0
        maxval = 0
        maxnum2500 = 0
        fullcount2500 = 0
        t_if = True
        tcheck = 0
        for i in range(timebins):
            for j in range(0, tmp + 1):
                if maxval < ims[i][j]:
                    maxval = ims[i][j]
                    ti = i
                if ims[i][j] >= 200:
                    maxnum2500 += 1
                    fullcount2500 += ims[i][j]
                    if t_if != i:
                        t_if = i
                        tcheck += 1

        cry_bindo = tcheck/timebins  * 100
        cry_volum = maxnum2500/timebins * 100
        if cry_volum > 10:
            result = "cry"

        if tcheck/timebins  * 100 < 20:
            result += ", sleepy"
        else :
            if cry_volum < 60:
                result += ", sick"
            else:
                result += ", hungry"
        
                            
        self.mPush.insertMSG("-----------------vo thread : (%s)" % result)

        return result